@@ -5,10 +5,11 @@ use warnings;
 
 use parent qw{ HTML::Parser };
 
-our $VERSION = '0.199_01';
+our $VERSION = '0.200_01';
 
 our $DEBUG = 0;
 
+my @stacked = qw{ current_table current_row current_element };
 
 sub start
 {
@@ -20,13 +21,16 @@ sub start
 # Store the incoming details in the current 'object'.
 	if ($tag eq 'table') {
 		my $table = $attr;
+		push @{ $self->{STORE}{stack} }, {
+			map { $_ => $self->{STORE}{$_} } @stacked };
 		push @{$self->{STORE}->{tables}}, $table;
 		$self->{STORE}->{current_table} = $table;
+		$self->{STORE}->{current_row} = undef;
+		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'th') {
 		my $th = $attr;
 		push @{$self->{STORE}->{current_table}->{headers}}, $th;
-		$self->{STORE}->{current_header} = $th;
 		$self->{STORE}->{current_element} = $th;
 
 	} elsif ($tag eq 'tr') {
@@ -38,7 +42,6 @@ sub start
 	} elsif ($tag eq 'td') {
 		my $td = $attr;
 		push @{$self->{STORE}->{current_row}->{cells}}, $td;
-		$self->{STORE}->{current_data_cell} = $td;
 		$self->{STORE}->{current_element} = $td;
 
 	} elsif ($tag eq 'caption') {
@@ -54,9 +57,9 @@ sub start
 			$self->_debug('TEXT(tag) = ', $origtext) if $DEBUG;
 			$elem->{data} .= $origtext;
 		}
-		
+
 	}
-	
+
 	$self->_debug($origtext) if $DEBUG;
 
 	return;
@@ -87,27 +90,18 @@ sub end
 
 # Turn off the current object
 	if ($tag eq 'table') {
-		$self->{STORE}->{current_table} = undef;
-		$self->{STORE}->{current_row} = undef;
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
-		$self->{STORE}->{current_element} = undef;
+		my $prev = pop @{ $self->{STORE}{stack} } || [];
+		$self->{STORE}{$_} = $prev->{$_} for @stacked;
 
 	} elsif ($tag eq 'th') {
 		$self->{STORE}->{current_row} = undef;
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
 		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'tr') {
 		$self->{STORE}->{current_row} = undef;
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
 		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'td') {
-		$self->{STORE}->{current_data_cell} = undef;
-		$self->{STORE}->{current_header} = undef;
 		$self->{STORE}->{current_element} = undef;
 
 	} elsif ($tag eq 'caption') {
@@ -121,7 +115,7 @@ sub end
 			$self->_debug('TEXT(tag) = ', $origtext) if $DEBUG;
 			$elem->{data} .= $origtext;
 		}
-		
+
 	}
 
 	$self->_debug($origtext) if $DEBUG;
@@ -134,16 +128,21 @@ sub parse
 {
 	my ($self, $data) = @_;
 
-	$self->{STORE} = undef;
+	unless ( defined $data ) {	# RT 7262
+	    require Carp;
+	    Carp::croak( 'Argument must be defined' );
+	}
 
-# Ensure the following keys exist
-	$self->{STORE}->{current_data_cell} = undef;
-	$self->{STORE}->{current_row} = undef;
-	$self->{STORE}->{current_table} = undef;
+	$self->{STORE} = {
+	    stack	=> [],
+	};
 
 	$self->SUPER::parse($data);
 
-	return $self->{STORE}->{tables};
+	my $tables = $self->{STORE}{tables};
+	delete $self->{STORE};
+
+	return $tables;
 }
 
 
@@ -178,7 +177,13 @@ HTML::TableContentParser - Do interesting things with the contents of tables.
 This package pulls out the contents of a table from a string containing HTML.
 Each time a table is encountered, data will be stored in an array consisting
 of a hash of whatever was discovered about the table -- id, name, border,
-cell spacing etc, and of course data contained within the table. 
+cell spacing etc, and of course data contained within the table.
+
+Tables appear in the output in the order in which they are encountered.
+If a table is nested inside a cell of another table, it will appear
+after the containing table in the output, and any connection between the
+two will be lost. As of version 0.200_01, the appearance of a nested
+table should not cause any truncation of the containing table.
 
 The format of each hash will look something like
 
@@ -186,7 +191,7 @@ The format of each hash will look something like
   @{$table_headers}     array of table headers, in order found
   @{$table_rows}        rows discovered, in order
 
-If the table has a caption, this will be provided as 
+If the table has a caption, this will be provided as
 
   caption               keys from the caption tag's attributes
     data                the text of the <caption>..</caption> element
@@ -198,7 +203,7 @@ then for each table row,
 then for each data cell,
       data              what comes between <td> and </td>
       other attributes  the ... in <td ...>
-		  
+
 
 =head2 EXAMPLE
 
@@ -210,13 +215,13 @@ then for each data cell,
     for $r (@{$t->{rows}}) {
 			print "Row: ";
       for $c (@{$r->{cells}}) {
-        print "[$c->{data}] ";				
-      }				
-      print "\n";			
+        print "[$c->{data}] ";
+      }
+      print "\n";
     }
   }
 
-	
+
 =head1 METHODS
 
 =over 4
@@ -227,22 +232,25 @@ Called whenever a particular start tag has been recognised. This is called
 automatically by the parser and should not be called from the application.
 
 
-=item text($parser, $content); 
+=item text($parser, $content);
 
 Called whenever a piece of content is encountered. This is called
 automatically by the parser and should not be called from the application.
 
-=item end($parser, $tag, $origtext);  
+=item end($parser, $tag, $origtext);
 
 Called whenever a particular end tag is encountered. This is called
 automatically by the parser and should not be called from the application.
 
 =item $tables_ref = $p->parse($html);
 
-Called with the HTML to parse. This is all the application needs to do. 
+Called with the HTML to parse. This is all the application needs to do.
 The return value will be an arrayref containing each table encountered, in the
 format detailed above.
 
+This method will C<croak()> if the argument is not defined, or not
+specified.
+
 =item DEBUG
 
 Not a method, but a class variable. Set to 1 to cause debugging output
@@ -263,7 +271,7 @@ Nothing.
 This module is a very specific tool to address a very specific problem.
 One of the following modules may better address your needs.
 
-L<HTML::Parser|HTML::Parser>. This is a general HTML parser, which form
+L<HTML::Parser|HTML::Parser>. This is a general HTML parser, which forms
 the basis for this module.
 
 L<HTML::TreeBuilder|HTML::TreeBuilder>. This is a general HTML parser,
@@ -283,7 +291,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 Copyright (C) 2002 Simon Drabble
 
-Copyright (C) 2017 Thomas R. Wyant, III
+Copyright (C) 2017-2018 Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
